package lsedit;

import java.util.*;

import java.awt.*;



public class StringLinizer extends Object {

    protected FontMetrics fm;
    protected int         width;

    protected String	  input;
    protected char[]	  inputchars;
    protected int	  start;
    protected int	  lth;

    // Exported routines

    
    public StringLinizer(String str, FontMetrics fm, int width) {

	this.fm    = fm;
	this.width = width;
	this.input = str;
	start      = 0;
	lth        = str.length();
	inputchars = input.toCharArray();

    }


    public boolean hasMoreLines()
    {
	return (start < lth);
    }

    public String nextLine() 
    {
	String	rest;
	int	at, best, state, subwidth;

	if (fm.stringWidth(input.substring(start)) <= width) {
	    // Simple case -- rest of string fits in one line
	    at = lth;
	} else {

	    // Normal case break at some sort of word boundary

	    best  = lth-1;
	    state = 0;
	    for (at = best; at > start;) {
		switch (inputchars[--at]) {
		case '\r':
		case '<':
		    state = 1;	// This can start a new line
		    continue;
		case '\n':
		case '\t':
		case ' ':
		case '=':
		case '"':
		case '>':
		case '-':
		    state = 1;		// This can terminate a line or start a new line
		    break;
		default:
		    if (state != 0) {	// If next character may start a new line
			state = 0;	// this character may terminate a line
			break;
		    }
		    continue;
		}
		subwidth = fm.charsWidth(inputchars, start, at+1-start);

	        if (subwidth <= width) {
		    ++at;
		    break;
	    	}
		best = at;
	    }
	    if (at <= start) {
		// Bad case -- can't fit one word onto a line (fit as much as possible)

		for (at = best; --at > start;) {
		    subwidth = fm.charsWidth(inputchars, start, at-start);
		    if (subwidth <= width) {
			break;
	    }	}   }

	    if (at <= start) {
		// Terrible case can't fit a single character to a line
		// Discard the rest of the string and return an empty string

		start = lth;
		return("");	
	}   }

	state = 0;
	for (best = start; best < at; ++best) {
	    if (inputchars[best] == '\\' && inputchars[best+1] == 'n') {
		at    = best;
		state = 2;
		break;
	}   }

	rest  = input.substring(start, at);

	// Lines don't start with whitespace

	for (start = at+state; start < lth; ++start) {
	    switch (inputchars[start]) {
	    case ' ':
	    case '\t':
	    case '\n':
	        continue;
	    }
	    break;
	}
  	return(rest);
    } 
}

